package com.loafle.overflow.central.module.infra.service;

import com.loafle.overflow.central.commons.model.PageParams;
import com.loafle.overflow.central.commons.utils.PageUtil;
import com.loafle.overflow.central.module.domain.model.Domain;
import com.loafle.overflow.central.module.infra.dao.InfraDAO;
import com.loafle.overflow.central.module.probe.exception.ProbeNotFoundException;
import com.loafle.overflow.model.probe.Probe;
import com.loafle.overflow.central.module.probe.service.ProbeService;
import com.loafle.overflow.central.module.sensor.dao.SensorDAO;
import com.loafle.overflow.model.target.Target;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.infra.Infra;
import com.loafle.overflow.model.infra.InfraService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * Created by insanity on 17. 6. 28.
 */

@Service("InfraService")
public class CentralInfraService implements InfraService {

    @Autowired
    InfraDAO infraDAO;

    @Autowired
    SensorDAO sensorDAO;

    @Autowired
    private ProbeService probeService;

    public Infra regist(Infra infra) throws OverflowException {
        return this.infraDAO.save(infra);
    }

    public Infra read(long id) throws OverflowException {
        Infra infra = this.infraDAO.findOne(id);
        infra.getTarget().setSensors(this.sensorDAO.findAllByTarget(infra.getTarget()));
        return infra;
    }

    public Page<Infra> readAllByProbe(Probe probe, PageParams pageParams) throws OverflowException {
        return this.infraDAO.findAllByProbe(probe, PageUtil.getPageRequest(pageParams));
    }

    public Page<Infra> readAllByDomain(Domain domain, PageParams pageParams) throws OverflowException {
        List<Probe> probeList = this.probeService.readAllByDomain(domain);

        if(probeList == null || probeList.size() <= 0) {
            throw new ProbeNotFoundException();
        }

        Page<Infra> infraList = this.infraDAO.findAllByProbeList(probeList, PageUtil.getPageRequest(pageParams));
        for (Infra infra: infraList) {
            infra.getTarget().setSensors(this.sensorDAO.findAllByTarget(infra.getTarget()));
        }
        return infraList;
    }


    public List<Target> readAllTargetByDomain(Domain domain) throws OverflowException {

        List<Probe> probeList = this.probeService.readAllByDomain(domain);

        if(probeList == null || probeList.size() <= 0) {
            throw new ProbeNotFoundException();
        }

        return this.infraDAO.findAllTargetByProbeList(probeList);
    }

    public List<Target> readAllTargetByProbeList(List<Probe> probeList) throws OverflowException {
        return this.infraDAO.findAllTargetByProbeList(probeList);
//        return null;
    }

    public Infra readByTarget(Target target) throws OverflowException {
        return this.infraDAO.findByTarget(target);
    }

}
